﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Forms;
using System.Reflection;
using System.Workflow.Activities;

namespace WorkflowSimulator
{
    /// <summary>
    /// This page describes the layout of the main workflow simulation user interface.
    /// </summary>
    public partial class WorkflowSimulationHost : Window
    {
        public WorkflowSimulationHost()
        {
            InitializeComponent();
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            base.OnClosing(e);

            if (workflowDiagram.WorkflowRuntime != null)
            {
                workflowDiagram.WorkflowRuntime.StopRuntime();
            }
        }

        private void WorkflowDiagram_ActivitySelected(object sender, System.ComponentModel.Design.ComponentEventArgs e)
        {
            if (e.Component != null)
            {
                workflowStatusBar.ItemsSource = new string[] { String.Format("Current activity: {0}", e.Component.Site.Name) };
            }
            else
            {
                workflowStatusBar.ItemsSource = new string[] { "" };
            }
        }

        private void fileMenuOpenWorkflow_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog openFileDialog = new Microsoft.Win32.OpenFileDialog();
            openFileDialog.Filter = "Workflow Assemblies (*.dll)|*.dll|All Files (*.*)|*.*";
            openFileDialog.InitialDirectory = AppDomain.CurrentDomain.BaseDirectory;

            if (openFileDialog.ShowDialog().GetValueOrDefault())
            {
                try
                {
                    LoadWorkflow(openFileDialog.FileName);
                }
                catch (Exception ex)
                {
                    System.Windows.MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        private void LoadWorkflow(string workflowFileName)
        {
            Type uiType = null;
            Type localServiceType = null;
            Type workflowType = null;

            if (System.IO.Path.GetDirectoryName(workflowFileName) != System.IO.Path.GetDirectoryName(AppDomain.CurrentDomain.BaseDirectory))
            {
                throw new System.Exception("Workflow assemblies and dependencies must be copied into the same directory as the Workflow Simulator and opened from there.");
            }

            //Currently user selected assemblies load into the LoadFrom context,
            //which means later requests which use probing to load the assembly for
            //execution have to load from the same path or an error will be thrown.
            Assembly asm = Assembly.LoadFrom(workflowFileName);

            Type[] availableTypes = asm.GetTypes();

            var workflowTypes = from t in availableTypes
                                where typeof(SequentialWorkflowActivity).IsAssignableFrom(t) ||
                                      typeof(StateMachineWorkflowActivity).IsAssignableFrom(t)
                                select t;

            if (workflowTypes.Count() > 1)
            {
                //TODO: Multiple workflows might be available, eventually may need to ask user.
                //for now, just load the first one.
                workflowType = workflowTypes.First();
            }
            else if (workflowTypes.Count() == 1)
            {
                workflowType = workflowTypes.Single();
            }
            else
            {
                throw new Exception("The selected assembly does not contain any workflow definitions.");
            }
            
            WorkflowSimulationUIAttribute[] simulationAttributes = (WorkflowSimulationUIAttribute[])workflowType.GetCustomAttributes(typeof(WorkflowSimulationUIAttribute), true);

            if (simulationAttributes.Length == 1)
            {
                localServiceType = Type.GetType(simulationAttributes[0].LocalServiceType, false);
                uiType = Type.GetType(simulationAttributes[0].UserInterfaceType, false);
            }

            workflowDiagram.LocalServiceType = localServiceType;
            workflowDiagram.WorkflowType = workflowType;

            workflowDiagram.DisplayWorkflow();

            workflowDiagram.ShowAECCounter = chkShowAecCounters.IsChecked.Value;

            SetSimulationSpeed();

            LoadUserInterface(uiType);
        }

        private void LoadUserInterface(Type uiType)
        {
            stackPanelUIHost.Children.Clear();
            IWorkflowUI workflowUI;

            if (uiType == null)
            {
                //Load a default UI that only lets user run workflow
                workflowUI = new RunWorkflowUI();

            }
            else
            {
                //Load the custom UI appropriate for driving the workflow
                workflowUI = (IWorkflowUI)uiType.GetConstructor(Type.EmptyTypes).Invoke(null);
            }

            workflowUI.WorkflowDiagram = this.workflowDiagram;
            stackPanelUIHost.Children.Add((UIElement)workflowUI);
        }

        private void fileMenuExit_Click(object sender, RoutedEventArgs e)
        {
            App.Current.Shutdown();
        }

        private void cboSpeed_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            SetSimulationSpeed();
        }

        private void SetSimulationSpeed()
        {
            if (workflowDiagram != null)
            {
                if (cboSpeed.SelectedValue.ToString() == "Full Speed")
                {
                    workflowDiagram.ActivityRefreshDelay = 0;
                }
                else if (cboSpeed.SelectedValue.ToString() == "Fast")
                {
                    workflowDiagram.ActivityRefreshDelay = 750;
                }
                else if (cboSpeed.SelectedValue.ToString() == "Slow")
                {
                    workflowDiagram.ActivityRefreshDelay = 1000;
                }
            }
        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if (workflowDiagram != null)
                workflowDiagram.ShowAECCounter = true;
        }

        private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            if (workflowDiagram != null)
                workflowDiagram.ShowAECCounter = false;
        }

        
    }
}
