﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime.Tracking;
using System.Drawing;
using System.Workflow.ComponentModel;

namespace WorkflowSimulator
{
    public class SimulationGlyphProvider : IDesignerGlyphProvider
    {
        /// <summary>
        /// Controls the display of the Activity Execution Context (AEC) counter overlaid over 
        /// the glyph of an executing activity. True enables the display.
        /// </summary>
        public bool ShowAecInfo
        {
            get;
            set;
        }

        public Dictionary<string, ActivityStatus> LatestActivityStatus
        {
            get;
            set;
        }

        public SimulationGlyphProvider()
        { }

        /// <summary>
        /// Returns the collection of glyphs to apply to the provided ActivityDesigner (and 
        /// its underlying Activity).
        /// </summary>
        /// <param name="activityDesigner"></param>
        /// <returns></returns>
        ActivityDesignerGlyphCollection IDesignerGlyphProvider.GetGlyphs(ActivityDesigner activityDesigner)
        {
            ActivityDesignerGlyphCollection glyphList = new ActivityDesignerGlyphCollection();

            try
            {
                if (LatestActivityStatus != null &&
                    LatestActivityStatus.ContainsKey(activityDesigner.Activity.QualifiedName))
                {
                    ActivityStatus status = LatestActivityStatus[activityDesigner.Activity.QualifiedName];

                    if (status.ActivityTrackingRecords.Last().ExecutionStatus == ActivityExecutionStatus.Executing)
                    {
                        glyphList.Add(new ExecutingGlyph(status, ShowAecInfo));
                    }
                    else if (status.ActivityTrackingRecords.Last().ExecutionStatus == ActivityExecutionStatus.Closed)
                    {
                        glyphList.Add(new CompletedGlyph(status, ShowAecInfo));
                    }
                    
                }


            }
            catch (Exception e)
            {
                System.Diagnostics.Debug.WriteLine("Unable to set activity glyph. " + e.ToString());
            }

            return glyphList;
        }
    }

    public class ActivityStatus
    {
        public List<ActivityTrackingRecord> ActivityTrackingRecords
        {
            get;
            set;
        }
    }

    internal class ExecutingGlyph : DesignerGlyph
    {
        ActivityStatus _activityStatus = null;
        bool _showAecInfo = false;

        internal ExecutingGlyph(ActivityStatus activityStatus, bool showAecInfo)
        {
            _activityStatus = activityStatus;
            _showAecInfo = showAecInfo;
        }

        public override Rectangle GetBounds(ActivityDesigner designer, bool activated)
        {
            Rectangle bounds = designer.Bounds;
            bounds.Inflate((int)(WorkflowTheme.CurrentTheme.AmbientTheme.GlyphSize.Width * 0.75), 
                (int)(WorkflowTheme.CurrentTheme.AmbientTheme.GlyphSize.Height * 0.75));

            return bounds;
        }

        /// <summary>
        /// Draws a yellow bounding box around the exeucting activity. Optionally draws
        /// the AEC counter in the top, left corner of the activity glyph.
        /// </summary>
        protected override void OnPaint(Graphics graphics, bool activated, AmbientTheme ambientTheme, ActivityDesigner designer)
        {
            Rectangle bounds = designer.Bounds;
            bounds.Inflate((int)(ambientTheme.GlyphSize.Width * 0.75), (int)(ambientTheme.GlyphSize.Height * 0.75));
            graphics.DrawRectangle(new Pen(Color.Yellow, 5), designer.Bounds);

            if (_showAecInfo)
            {
                graphics.DrawString(String.Format("{0} AEC's", _activityStatus.ActivityTrackingRecords.Where(rec => rec.ExecutionStatus == ActivityExecutionStatus.Executing).Count()),
                                    ambientTheme.Font,
                                    ambientTheme.ForegroundBrush,
                                    designer.Bounds.Left + 3,
                                    designer.Bounds.Top + 3);
            }
        }
    }

    internal class CompletedGlyph : DesignerGlyph
    {
        ActivityStatus _activityStatus = null;
        bool _showAecInfo = false;

        internal CompletedGlyph(ActivityStatus activityStatus, bool showAecInfo)
        {
            _activityStatus = activityStatus;
            _showAecInfo = showAecInfo;
        }

        public override Rectangle GetBounds(ActivityDesigner designer, bool activated)
        {
            Rectangle imageBounds = Rectangle.Empty;
            Image image = WorkflowSimulator.Properties.Resources.complete;
            if (image != null)
            {
                Size glyphSize = WorkflowTheme.CurrentTheme.AmbientTheme.GlyphSize;
                imageBounds.Location = new Point(designer.Bounds.Right - glyphSize.Width / 2, designer.Bounds.Top + glyphSize.Height / 2);
                imageBounds.Size = glyphSize;
            }
            return imageBounds;
        }

        protected override void OnPaint(Graphics graphics, bool activated, AmbientTheme ambientTheme, ActivityDesigner designer)
        {
            Bitmap bitmap = WorkflowSimulator.Properties.Resources.complete;
            bitmap.MakeTransparent(Color.FromArgb(0, 255, 255));
            if (bitmap != null)
                graphics.DrawImage(bitmap, GetBounds(designer, activated), new Rectangle(Point.Empty, bitmap.Size), GraphicsUnit.Pixel);

            if (_showAecInfo)
            {
                graphics.DrawString(String.Format("{0} AEC's", _activityStatus.ActivityTrackingRecords.Where(rec => rec.ExecutionStatus == ActivityExecutionStatus.Executing).Count()),
                                    ambientTheme.Font,
                                    ambientTheme.ForegroundBrush,
                                    designer.Bounds.Left + 3,
                                    designer.Bounds.Top + 3);
            }
        }
    }
}
