﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Workflow.Runtime.Tracking;
using System.Workflow.ComponentModel;

namespace WorkflowSimulator
{
    public class SimulationTrackingService: TrackingService
    {
        public event EventHandler<ActivityTrackingEventArgs> ActivityStatusUpdate;

        protected override TrackingProfile GetProfile(Guid workflowInstanceId)
        {
            return SimulationTrackingService.GetUnfilteredProfile();
        }

        protected override TrackingProfile GetProfile(Type workflowType, Version profileVersionId)
        {
            return SimulationTrackingService.GetUnfilteredProfile();
        }

        protected override TrackingChannel GetTrackingChannel(TrackingParameters parameters)
        {
            SimulationTrackingChannel simulationTrackingChannel = new SimulationTrackingChannel(parameters);
            simulationTrackingChannel.ActivityStatusUpdate += new EventHandler<ActivityTrackingEventArgs>(simulationTrackingChannel_ActivityStatusUpdate);
            
            return simulationTrackingChannel;
        }

        protected void simulationTrackingChannel_ActivityStatusUpdate(object sender, ActivityTrackingEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Received status update for: " + e.ActivityTrackingRecord.QualifiedName);

            OnActivityStatusUpdate(e);
        }

        protected void OnActivityStatusUpdate(ActivityTrackingEventArgs activityTrackingRecordEventArgs)
        {
            if (ActivityStatusUpdate != null)
            {
                ActivityStatusUpdate(this, activityTrackingRecordEventArgs);
            }
        }

        protected override bool TryGetProfile(Type workflowType, out TrackingProfile profile)
        {
            profile = SimulationTrackingService.GetUnfilteredProfile();
            return true;
        }

        protected override bool TryReloadProfile(Type workflowType, Guid workflowInstanceId, out TrackingProfile profile)
        {
            profile = null;
            return false;
        }

        private static TrackingProfile GetUnfilteredProfile()
        {
            TrackingProfile profile = new TrackingProfile();
            profile.Version = new Version("1.0.0");

            ActivityTrackPoint activityTrackPoint = GetTrackPointForAllActivityEvents();
            profile.ActivityTrackPoints.Add(activityTrackPoint);

            WorkflowTrackPoint workflowTrackPoint = GetTrackPointForAllWorkflowEvents();
            profile.WorkflowTrackPoints.Add(workflowTrackPoint);

            UserTrackPoint userTrackPoint = GetTrackPointForAllUserEvents();
            profile.UserTrackPoints.Add(userTrackPoint);
            return profile;
        }

        private static UserTrackPoint GetTrackPointForAllUserEvents()
        {
            UserTrackPoint userTrackPoint = new UserTrackPoint();
            UserTrackingLocation userLocation = new UserTrackingLocation();
            userLocation.ActivityType = typeof(Activity);
            userLocation.MatchDerivedActivityTypes = true;
            userLocation.ArgumentType = typeof(object);
            userLocation.MatchDerivedArgumentTypes = true;
            userTrackPoint.MatchingLocations.Add(userLocation);
            return userTrackPoint;
        }

        private static WorkflowTrackPoint GetTrackPointForAllWorkflowEvents()
        {
            WorkflowTrackPoint workflowTrackPoint = new WorkflowTrackPoint();
            workflowTrackPoint.MatchingLocation = new WorkflowTrackingLocation();
            foreach (TrackingWorkflowEvent workflowEvent in Enum.GetValues(typeof(TrackingWorkflowEvent)))
            {
                workflowTrackPoint.MatchingLocation.Events.Add(workflowEvent);
            }
            return workflowTrackPoint;
        }

        private static ActivityTrackPoint GetTrackPointForAllActivityEvents()
        {
            ActivityTrackPoint activityTrackPoint = new ActivityTrackPoint();
            ActivityTrackingLocation activityLocation = new ActivityTrackingLocation(typeof(Activity));
            activityLocation.MatchDerivedTypes = true;
            foreach (ActivityExecutionStatus status in Enum.GetValues(typeof(ActivityExecutionStatus)))
            {
                activityLocation.ExecutionStatusEvents.Add(status);
            }
            activityTrackPoint.MatchingLocations.Add(activityLocation);
            return activityTrackPoint;
        }
    }
}
