﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.ComponentModel.Design;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Tracking;
using System.Workflow.Activities;
using System.Collections.Specialized;

namespace WorkflowSimulator
{
    public partial class WorkflowDiagram : UserControl
    {
        private DesignSurface _designSurface;
        private ITypeProvider _typeProvider;
        private WorkflowDefinitionLoader _workflowDefinitionLoader;
        private WorkflowView _workflowView;
        private Activity _activeWorkflowInstance;
        private object _localServiceInstance;
        private SimulationGlyphProvider _glyphProvider;

        public event EventHandler<EventArgs> AfterCreateWorkflowRuntime;

        public Dictionary<string, ActivityStatus> LatestActivityStatus
        {
            get;
            set;
        }

        public Type LocalServiceType
        {
            get;
            set;
        }
        
        public WorkflowRuntime WorkflowRuntime
        {
            get;
            set;
        }

        public Type WorkflowType
        {
            get;
            set;
        }

        public object LocalServiceInstance
        {
            get
            {
                return _localServiceInstance;
            }
        }

        public WorkflowInstance WorkflowInstance
        {
            get;
            set;
        }

        public int ActivityRefreshDelay
        {
            get;
            set;
        }

        public bool ShowAECCounter
        {
            get
            {
                if (_glyphProvider != null)
                {
                    return _glyphProvider.ShowAecInfo;
                }
                else return false;
            }
            set
            {
                if (_glyphProvider != null)
                {
                    _glyphProvider.ShowAecInfo = value;
                }
            }
        }

        public WorkflowDiagram()
        {
            InitializeComponent();

            _typeProvider = new TypeProvider(new ServiceContainer());
            _workflowDefinitionLoader = new WorkflowDefinitionLoader();
            _workflowDefinitionLoader.TypeProvider = _typeProvider;

            LatestActivityStatus = new Dictionary<string, ActivityStatus>();
        }

        private void InitializeWorkflowRuntime()
        {
            WorkflowRuntime = new WorkflowRuntime();
            WorkflowRuntime.WorkflowStarted += new EventHandler<WorkflowEventArgs>(WorkflowRuntime_WorkflowStarted);
            WorkflowRuntime.WorkflowCompleted += new EventHandler<WorkflowCompletedEventArgs>(WorkflowRuntime_WorkflowCompleted);
            WorkflowRuntime.WorkflowAborted += new EventHandler<WorkflowEventArgs>(WorkflowRuntime_WorkflowAborted);
            WorkflowRuntime.WorkflowTerminated += new EventHandler<WorkflowTerminatedEventArgs>(WorkflowRuntime_WorkflowTerminated);
            WorkflowRuntime.WorkflowIdled += new EventHandler<WorkflowEventArgs>(WorkflowRuntime_WorkflowIdled);
            
            SimulationTrackingService simulationTrackingService = new SimulationTrackingService();
            simulationTrackingService.ActivityStatusUpdate += new EventHandler<ActivityTrackingEventArgs>(simulationTrackingService_ActivityStatusUpdate);
            
            WorkflowRuntime.AddService(simulationTrackingService);

            if (LocalServiceType != null)
            {
                ExternalDataExchangeService dex = new ExternalDataExchangeService();
                WorkflowRuntime.AddService(dex);
                _localServiceInstance = LocalServiceType.GetConstructor(Type.EmptyTypes).Invoke(null);
                dex.AddService(this.LocalServiceInstance);
            }

            if (AfterCreateWorkflowRuntime != null)
            {
                AfterCreateWorkflowRuntime(this, new EventArgs());
            }
        }

        void WorkflowRuntime_WorkflowIdled(object sender, WorkflowEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(String.Format("Workflow '{0}' went idle.", e.WorkflowInstance.InstanceId));
        }

        void WorkflowRuntime_WorkflowAborted(object sender, WorkflowEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(String.Format("Workflow '{0}' was aborted.", e.WorkflowInstance.InstanceId));
        }

        void WorkflowRuntime_WorkflowCompleted(object sender, WorkflowCompletedEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(String.Format("Workflow '{0}' was completed.", e.WorkflowInstance.InstanceId));
        }

        void WorkflowRuntime_WorkflowStarted(object sender, WorkflowEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(String.Format("Workflow '{0}' was started.", e.WorkflowInstance.InstanceId));
        }

        void WorkflowRuntime_WorkflowTerminated(object sender, WorkflowTerminatedEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(String.Format("Workflow '{0}' was terminated. Exception:\r\n{1} ", 
                                                e.WorkflowInstance.InstanceId, e.Exception));
        }

        #region WorkflowView Designer Hosting

        void simulationTrackingService_ActivityStatusUpdate(object sender, ActivityTrackingEventArgs e)
        {
            lock (this)
            {
                UpdateGlyphs(e.ActivityTrackingRecord);

                if (e.ActivityTrackingRecord.ExecutionStatus == ActivityExecutionStatus.Executing)
                {
                    SelectActivity(e.ActivityTrackingRecord.QualifiedName);
                }

                System.Threading.Thread.Sleep(this.ActivityRefreshDelay);
            }
        }

        private void UpdateGlyphs(ActivityTrackingRecord activityTrackingRecord)
        {
            string activityQualifiedName = activityTrackingRecord.QualifiedName;

            if (LatestActivityStatus.ContainsKey(activityQualifiedName))
            {
                LatestActivityStatus[activityQualifiedName].ActivityTrackingRecords.Add(activityTrackingRecord);
            }
            else
            {
                LatestActivityStatus.Add(activityQualifiedName, new ActivityStatus()
                {
                    ActivityTrackingRecords = new List<System.Workflow.Runtime.Tracking.ActivityTrackingRecord>()
                    { 
                        activityTrackingRecord
                    }
                });
            }

            _workflowView.Invalidate();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
        }

        public void DisplayWorkflow()
        {
            if (WorkflowType == null)
            {
                return;
            }

            Activity definition = (Activity)WorkflowType.GetConstructor(Type.EmptyTypes).Invoke(null);

            DisplayWorkflow(definition);
        }

        public void DisplayWorkflow(Activity loadedDefinition)
        {
            if (loadedDefinition == null)
            {
                return;
            }

            ResetDisplay();

            _activeWorkflowInstance = loadedDefinition;

            _designSurface = new DesignSurface();
            _workflowDefinitionLoader.WorkflowDefinition = _activeWorkflowInstance;

            _designSurface.BeginLoad(_workflowDefinitionLoader);

            IDesignerHost designerHost = _designSurface.GetService(typeof(IDesignerHost)) as IDesignerHost;
            if (designerHost == null || designerHost.RootComponent == null)
            {
                throw new ApplicationException("Failed to load workflow. Either designer host or root component was null.");
            }

            IRootDesigner rootDesigner = designerHost.GetDesigner(designerHost.RootComponent) as IRootDesigner;
            if (rootDesigner != null)
            {
                SuspendLayout();

                ConfigureWorkflowView();
                RegisterSelectionService();
                RegisterGlyphProvider();
                ConfigureMenuCommandService(designerHost);

                ResumeLayout();
            }
        }

        public IDesignerHost GetDesignerHost()
        {
            return _designSurface.GetService(typeof(IDesignerHost)) as IDesignerHost;
        }

        private void RegisterSelectionService()
        {
            ISelectionService selectionService = ((IServiceProvider)_workflowView).GetService(typeof(ISelectionService)) as ISelectionService;
            if (selectionService != null)
            {
                selectionService.SelectionChanged += new EventHandler(selectionService_SelectionChanged);
            }
        }

        private void RegisterGlyphProvider()
        {
            IDesignerGlyphProviderService glyphService = _designSurface.GetService(typeof(IDesignerGlyphProviderService)) as IDesignerGlyphProviderService;
            _glyphProvider = new SimulationGlyphProvider();
            this.LatestActivityStatus.Clear();
            _glyphProvider.LatestActivityStatus = this.LatestActivityStatus;
            glyphService.AddGlyphProvider(_glyphProvider); 
        }

        private void ConfigureMenuCommandService(IDesignerHost designerHost)
        {
            IMenuCommandService menuService = (IMenuCommandService)designerHost.GetService(typeof(IMenuCommandService));
            MenuCommand mc = new MenuCommand(new EventHandler(
                delegate(object sender, EventArgs e) { _workflowView.FitToScreenSize(); })
                , WorkflowMenuCommands.SizeToFit);
            menuService.AddCommand(mc);
        }

        public event EventHandler<ComponentEventArgs> ActivitySelected;

        void selectionService_SelectionChanged(object sender, EventArgs e)
        {
            ISelectionService selectionService = ((IServiceProvider)_workflowView).GetService(typeof(ISelectionService)) as ISelectionService;
            if (selectionService != null)
            {
                if (ActivitySelected != null)
                {
                    ComponentEventArgs args = new ComponentEventArgs((IComponent)selectionService.PrimarySelection);
                    ActivitySelected(this, args);
                }
            }
        }

        private void ConfigureWorkflowView()
        {
            _workflowView = _designSurface.View as WorkflowView;
            this.Controls.Add(_workflowView);
            _workflowView.Dock = DockStyle.Fill;
            _workflowView.Focus();
            _workflowView.FitToWorkflowSize();
        }

        public void ResetDisplay()
        {
            if (_designSurface != null)
            {
                IDesignerHost designerHost = _designSurface.GetService(typeof(IDesignerHost)) as IDesignerHost;
                if (designerHost != null)
                {
                    if (designerHost.Container.Components.Count > 0)
                    {
                        _workflowDefinitionLoader.RemoveFromDesigner(designerHost, designerHost.RootComponent as Activity);
                    }
                }
            }

            if (_designSurface != null)
            {
                _designSurface.Dispose();
                _designSurface = null;
            }

            if (_workflowView != null)
            {
                ISelectionService selectionSerivice = ((IServiceProvider)_workflowView).GetService(typeof(ISelectionService)) as ISelectionService;

                if (selectionSerivice != null)
                {
                    selectionSerivice.SelectionChanged -= new EventHandler(selectionService_SelectionChanged);
                }

                Controls.Remove(_workflowView);
                _workflowView.Dispose();
                _workflowView = null;
            }
        }

        #endregion

        #region WorkflowView Interaction

        public void RunWorkflow()
        {
            RunWorkflow(null);
        }

        public void RunWorkflow(Dictionary<string, object> initParameters)
        {
            if (WorkflowType == null)
            {
                return;
            }

            LatestActivityStatus.Clear();

            InitializeWorkflowRuntime();

            if (initParameters == null)
            {
                this.WorkflowInstance = WorkflowRuntime.CreateWorkflow(WorkflowType);
            }
            else
            {
                this.WorkflowInstance = WorkflowRuntime.CreateWorkflow(WorkflowType, initParameters);
            }

            this.WorkflowInstance.Start();
        }


        public void SelectActivity(string qualifiedName)
        {

            this.Invoke((MethodInvoker)delegate()
            {
                ISelectionService selectionService = _designSurface.GetService(typeof(ISelectionService)) as ISelectionService;

                if (selectionService != null)
                {
                    foreach (IComponent component in _designSurface.ComponentContainer.Components)
                    {
                        if (component.Site.Name == qualifiedName)
                        {
                            System.Collections.ICollection components = (System.Collections.ICollection)new IComponent[] { component };
                            selectionService.SetSelectedComponents(components, SelectionTypes.Primary);
                            _workflowView.EnsureVisible(selectionService.PrimarySelection);
                            break;
                        }
                    }
                }
            }
            );



         
        }

        #endregion
    }
}
